LIBRARY SQLLODBC 
DESCRIPTION 'SQLite ODBC Driver ' 

EXPORTS
SQLBulkOperations
SQLDataSources
SQLDrivers
SQLBrowseConnect
SQLPutData
SQLBindParameter
SQLBindParam
SQLNumParams
SQLParamData
SQLDescribeParam
SQLSetParam
SQLParamOptions
SQLTablePrivileges
SQLColumnPrivileges
SQLPrimaryKeys
SQLSpecialColumns
SQLForeignKeys
SQLEndTran
SQLTransact
SQLNativeSql
SQLProcedures
SQLProcedureColumns
SQLGetEnvAttr
SQLSetEnvAttr
SQLGetDiagRec
SQLGetDiagField
SQLGetStmtOption
SQLSetStmtOption
SQLSetPos
SQLSetScrollOptions
SQLGetInfo
SQLGetFunctions
SQLGetConnectAttr
SQLSetConnectAttr
SQLGetConnectOption
SQLSetConnectOption
SQLConnect
SQLDisconnect
SQLDriverConnect
SQLCancel
SQLGetCursorName
SQLSetCursorName
SQLCloseCursor
SQLBindCol
SQLTables
SQLColumns
SQLGetTypeInfo
SQLStatistics
SQLGetData
SQLFetch
SQLFetchScroll
SQLExtendedFetch
SQLRowCount
SQLNumResultCols
SQLDescribeCol
SQLColAttributes
SQLColAttribute
SQLError
SQLMoreResults
SQLPrepare
SQLExecute
SQLExecDirect
SQLAllocHandle
SQLFreeHandle
SQLGetStmtAttr
SQLSetStmtAttr
SQLAllocStmt
SQLFreeStmt
SQLAllocConnect
SQLFreeConnect
ConfigDSN
