LIBRARY SQLITEODBCU
DESCRIPTION 'SQLite ODBC (UTF-8) Driver ' 
SEGMENTS
DLL_TEXT PRELOAD
INIT_TEXT PRELOAD 
HEAPSIZE 1024 
EXPORTS
SQLBulkOperations
SQLDataSourcesW
SQLDriversW
SQLBrowseConnectW
SQLPutData
SQLBindParameter
SQLBindParam
SQLNumParams
SQLParamData
SQLDescribeParam
SQLSetParam
SQLParamOptions
SQLTablePrivilegesW
SQLColumnPrivilegesW
SQLPrimaryKeysW
SQLSpecialColumnsW
SQLForeignKeysW
SQLEndTran
SQLTransact
SQLNativeSqlW
SQLProceduresW
SQLProcedureColumnsW
SQLGetEnvAttr
SQLSetEnvAttr
SQLGetDiagRecW
SQLGetDiagFieldW
SQLGetStmtOptionW
SQLSetStmtOptionW
SQLGetStmtOption
SQLSetStmtOption
SQLSetPos
SQLSetScrollOptions
SQLGetInfoW
SQLGetFunctions
SQLGetConnectAttrW
SQLSetConnectAttrW
SQLGetConnectOptionW
SQLSetConnectOptionW
SQLConnectW
SQLDisconnect
SQLDriverConnectW
SQLCancel
SQLGetCursorNameW
SQLSetCursorNameW
SQLCloseCursor
SQLBindCol
SQLTablesW
SQLColumnsW
SQLGetTypeInfoW
SQLStatisticsW
SQLGetData
SQLFetch
SQLFetchScroll
SQLExtendedFetch
SQLRowCount
SQLNumResultCols
SQLDescribeColW
SQLColAttributesW
SQLColAttributeW
SQLErrorW
SQLMoreResults
SQLPrepareW
SQLExecute
SQLExecDirectW
SQLAllocHandle
SQLFreeHandle
SQLGetStmtAttrW
SQLSetStmtAttrW
SQLAllocStmt
SQLFreeStmt
SQLAllocConnect
SQLFreeConnect
ConfigDSN
install
uninstall
