#define DLG_SQLITEODBCCONFIGURATION 256
#define ST_DATASOURCENAME 258
#define ST_DESCRIPTION 259
#define ST_DATABASENAME 260
#define ST_TIMEOUT 261
#define EF_DSNNAME 262
#define EF_DSNDESC 263
#define EF_DATABASE 264
#define EF_TIMEOUT 265
#define IDC_BROWSE 266
#define IDC_NOWCHAR 267
#define IDC_STEPAPI 268
#define IDC_NOTXN 269
#define IDC_LONGNAM 270
